#include "cui.h"
#include "esc.h"
#include "key.h"
#include "handler.h"
#include "panel.h"
#include "label.h"
#include "button.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

cui cui_focus = NULL;
int cui_running = FALSE;

cui
cui_alloc(int size)
{
	cui obj;

	if((obj = malloc(size)) == NULL) ERR("No Mem");
	return obj;
}

void
cui_base_init(cui obj, cui parent, int x, int y, int w, int h)
{
	obj->parent = parent;
	obj->children = NULL;
	obj->next = NULL;
	obj->x = x;
	obj->y = y;
	obj->w = w;
	obj->h = h;

	if(parent) cui_base_child_add(parent, obj);
	obj->handler_list = NULL;
	obj->flags = 0;
}

void
cui_base_child_add(cui obj, cui child)
{
	child->next = obj->children;
	obj->children = child;
}

void
cui_draw_str(cui obj, int x, int y, char *s, int attr)
{
	cui_esc_loc(cui_gx(obj) + x, cui_gy(obj) + y);
	cui_esc_attr(attr);
	printf("%s", s);
	fflush(stdout);
}

int
cui_gx(cui obj)
{
	return obj == NULL ? 0 : cui_gx(obj->parent) + obj->x;
}

int
cui_gy(cui obj)
{
	return obj == NULL ? 0 : cui_gy(obj->parent) + obj->y;
}

int
cui_glen(cui o1, cui o2)
{
	int dx, dy;

	dx = (cui_gx(o1) + o1->w / 2) - (cui_gx(o2) + o2->w / 2);
	dy = (cui_gy(o1) + o1->h / 2) - (cui_gy(o2) + o2->h / 2);
	return dx * dx + dy * dy;
}

int
cui_focus_move_val(cui obj, int key)
{
	if(cui_focus) return cui_glen(obj, cui_focus);

	switch(key){
	case CUI_KEY_UP:
		return cui_gy(obj);
	case CUI_KEY_DOWN:
		return cui_gy(obj) + obj->h;
	case CUI_KEY_LEFT:
		return cui_gx(obj);
	case CUI_KEY_RIGHT:
		return cui_gx(obj) + obj->w;
	}
	return 0;	
}

cui
cui_focus_move_judge(cui obj, int key, cui old, int *val)
{
	int v, jg = FALSE;

	if(obj == NULL) return old;
	old = cui_focus_move_judge(obj->next, key, old, val);
	old = cui_focus_move_judge(obj->children, key, old, val);

	if(obj == cui_focus) return old;
	if(!(obj->flags & CUI_FLG_CAN_FOCUS)) return old;
	switch(key){
	case CUI_KEY_UP:
	case CUI_KEY_DOWN:
	case CUI_KEY_LEFT:
	case CUI_KEY_RIGHT:
		break;
	default:
		return old;
	}

	v = cui_focus_move_val(obj, key);
	if(!old){
		jg = TRUE;
	}else if(cui_focus){
		switch(key){
		case CUI_KEY_UP:
			jg = cui_gy(obj) <= cui_gy(cui_focus);
			break;
		case CUI_KEY_DOWN:
			jg = cui_gy(obj) >= cui_gy(cui_focus);
			break;
		case CUI_KEY_LEFT:
			jg = cui_gx(obj) <= cui_gx(cui_focus);
			break;
		case CUI_KEY_RIGHT:
			jg = cui_gx(obj) >= cui_gx(cui_focus);
			break;
		}
		jg = jg && v < *val;
	}else{
		switch(key){
		case CUI_KEY_UP:
		case CUI_KEY_LEFT:
			jg = v < *val;
			break;
		case CUI_KEY_DOWN:
		case CUI_KEY_RIGHT:
			jg = v > *val;
			break;
		}
	}
	if(jg) *val = v;
	return jg ? obj : old;
}

void
cui_draw(cui obj)
{
	int val;
	
	if(obj == NULL) return;
	val = (obj == cui_focus) ? CUI_DRAW_FOCUS : CUI_DRAW_NORMAL;
	cui_handler_call(obj, obj->handler_list, CUI_EVT_DRAW, val);

	cui_draw(obj->next);
	cui_draw(obj->children);
}

void
cui_quit(void)
{
	cui_running = FALSE;
}

void
cui_main(cui top_obj)
{
	int key, val;
	cui obj, bak;

	cui_esc_enter();
	cui_key_enter();

	cui_running = TRUE;	
	cui_draw(top_obj);
	while(cui_running){
		if((key = cui_key_get2()) == 0) continue;
		if(cui_focus){
			if(cui_handler_call(cui_focus, cui_focus->handler_list, CUI_EVT_KEY, key)) continue;
		}
		obj = cui_focus_move_judge(top_obj, key, NULL, &val);
		if(obj){
			bak = cui_focus;
			cui_focus = obj;
			cui_draw(bak);
			cui_draw(obj);
		}
	}
	cui_key_exit();
	cui_esc_exit();
}

void
cui_free(cui obj)
{
	if(obj == NULL) return;
	cui_free(obj->next);
	cui_free(obj->children);
	cui_handler_list_free(obj->handler_list);
	free(obj);
}

int
cui_simple_dialog_hdr(cui obj, int evt, int val, void *prm)
{
	char **result_ptr = (char **)prm;
	*result_ptr = cui_button_str_get(obj);
	cui_quit();
	return TRUE;
}

char *
cui_simple_dialog(int x, int y, char *s, char *s_btn1, char *s_btn2)
{
	int n, n1, n2, x2, w, w2;
	cui panel, btn1, btn2;
	char *btn_result;

	n = strlen(s);
	n1 = strlen(s_btn1);
	n2 = strlen(s_btn2);
	w = 3 + n + 3;
	x2 = 4 + 1 + n1 + 1 + 3;
	w2 = x2 + 1 + n2 + 1 + 3;
	w = w2 > w ? w2 : w;
	
	panel = cui_panel_new(NULL, x, y, w, 7);
	cui_label_new(panel, 3, 2, s);
	btn1 = cui_button_new(panel, 4, 5, s_btn1);
	btn2 = cui_button_new(panel, x2, 5, s_btn2);

	cui_bind(btn1, CUI_EVT_BUTTON, cui_simple_dialog_hdr, &btn_result);
	cui_bind(btn2, CUI_EVT_BUTTON, cui_simple_dialog_hdr, &btn_result);

	cui_main(panel);
	cui_free(panel);
	return btn_result;
}

/* EOF */
